(
macroScript OWO360Tool
category:"OWO360"
tooltip:"OWO360 Tool"
(
try destroyDialog OWO360_Rollout catch()

local OWOVer = "1.1.0"
local OWOFolder,OWODoIt	
local OWOCamTrans = matrix3 [0,-1,0] [0,0,1] [-1,0,0] [0,0,(units.decodeValue "1500mm")]
local OWOBoxTrans = matrix3 [1,0,0] [0,1,0] [0,0,1] [0,0,0]
local FBXParams = #("Animation" ,"ASCII" ,"AxisConversionMethod" ,"BakeAnimation" ,"BakeFrameStart" ,"BakeFrameEnd" ,"BakeFrameStep" ,"BakeResampleAnimation" ,"Cameras" ,"CAT2HIK" ,"ColladaTriangulate" ,"ColladaSingleMatrix" ,"ColladaFrameRate" ,"Convert2Tiff" ,"ConvertUnit" ,"EmbedTextures" ,"FileVersion" ,"FilterKeyReducer" ,"GeomAsBone" ,"GenerateLog" ,"Lights" ,"LoadExportPresetFile" ,"NormalsPerPoly" ,"PointCache" ,"PopSettings" ,"Preserveinstances" ,"PushSettings" ,"Removesinglekeys" ,"Resampling" ,"ResetExport" ,"ScaleFactor" ,"SelectionSet" ,"SelectionSetExport" ,"Shape" ,"Skin" ,"ShowWarnings" ,"SmoothingGroups" ,"SmoothMeshExport" ,"SplitAnimationIntoTakes" ,"TangentSpaceExport" ,"Triangulate" ,"UpAxis" ,"UseSceneName")
local FBXParamsVal = #(false, false, #animation, false, 0, 100, 1, false, false, false, true, true, 30.0, false, #MM, false, #FBX201800, false, true, true, false, unsupplied, false, false, unsupplied, false, unsupplied, false, 30.0, unsupplied, 1.0, unsupplied , false, true, true, true, false, false, unsupplied, false, false, #y, false)
local highVertsThresh = 8.0
local maxVer = (maxversion())[1]/1000
local jpegParamsVal = #(80,0)
local dist = 15.0
local paramOptimizeFThresh 	= #(4,8,10,50)


fn OWOFolder_check = 
(
OWOFolder = undefined 
	
	if maxFilePath != "" then --untitled
	(
	maxSceneName = getFilenameFile maxfileName
	OWOFolder = maxFilePath+("owo360-"+maxSceneName)
		
	if (isDirectoryWriteable OWOFolder) == false then makeDir OWOFolder
	)

OWOFolder	
)
fn setVertexColor node vert col = 
(
	if not polyop.getmapsupport node 0 do polyop.setmapsupport node 0 on
	polyop.setmapvert node 0 vert (col as point4)
)

fn collapseUtilSelected = 
(
UtilityPanel.OpenUtility collapse
children = windows.getChildrenHWND (windows.getDesktopHWND())
collapseButton = (for c in children where ( c[5] == "Collapse Selected" ) collect (c))[1]
UIAccessor.pressButton collapseButton[1]
)
	
fn removeModifier obj modClass =
(
	for theModifier in obj.modifiers where classOf theModifier == modClass do
		(
			deleteModifier obj theModifier
		)
)

fn setFBXParams =
(
	for p = 1 to FBXParams.count do
	(
	try FbxExporterSetParam FBXParams[p] FBXParamsVal[p] catch()
	)
--OpenFbxSetting() --debug	
)
fn exportToFBX fPathName=
(
--format "%\n" fPathName --debug
if (isDirectoryWriteable fPathName) == false then makeDir fPathName
	
exportFile fPathName #noPrompt selectedOnly:true using:FBXEXP
)
fn cameras_flt obj = isKindOf obj camera and (substring obj.name 1 4) != "owo_"

fn camerasOWO_flt obj = isKindOf obj camera and (substring obj.name 1 4) == "owo_"
	
fn checkValidUnits = 
(
	if units.SystemType == #millimeters then return true else return false
)
fn createOWOCamera camTrans cam: =
(
instCamOWO = ($'owo_Camera_*')[1]

	if cam == unsupplied then 
	(
		if maxVer < 18 then 
		(
		camOWO = freeCamera targetdistance:2000 transform:camTrans prefix:"owo_Camera_"
		)
		else
		(
		camOWO = physical_camera targeted:false target_distance:2000 transform:camTrans prefix:"owo_Camera_"
		)
	)
	else
	(
		if (superclassof cam) == camera then
		(
			case (classof cam) of
			(
			Physical: (
						camTarg = cam.targeted --bug 2019?
						cam.targeted = false --bug 2019?
						camOWO = copy cam
						camOWO.targeted = false 
						camOWO.target_distance = 2000
						camOWO.transform = camTrans
						camOWO.name = uniqueName "owo_Camera_"
						cam.targeted = camTarg --bug 2019?
						)
			default: (
						camOWO = copy cam
						camOWO.name = uniqueName "owo_Camera_"
					 )
			)
		)
	)
	if instCamOWO != undefined and camOWO != undefined then instanceReplace camOWO ($'owo_Camera_*'[1])
	camOWO.transform = camTrans
	
return camOWO
)
fn createOWOLayer =
(
OWOlayer = LayerManager.getLayerFromName "owo360"

	if OWOlayer == undefined then 
	(
	OWOlayer = LayerManager.newLayer()	
	OWOlayer.setname "owo360"
	)
)

fn addObjsToOWOLayer objs= 
(
OWOlayer = LayerManager.getLayerFromName "owo360"
for o = 1 to objs.count do OWOlayer.addnode objs[o]
)

fn addOWOToBatch cam folderPath =
(
	camNum = substring cam.name 12 cam.name.count
	owoName = "owo_"+camNum
	
	if (batchRenderMgr.FindView owoName) != 0 then batchRenderMgr.DeleteView (batchRenderMgr.FindView owoName)
		
	bView = batchRenderMgr.CreateView cam
	bView.name = owoName
	bView.outputFilename = folderPath+"\\"+camNum+".jpg"
	jpegio.setQuality jpegParamsVal[1]
	jpegio.setSmoothing jpegParamsVal[2]
	
)
fn createOWOBox cam =
(
	owoBoxName = substring cam.name 12 cam.name.count
	owoBox = box name:owoBoxName transform:OWOBoxTrans height:1 length:1 width:1 heightsegs:1 lengthsegs:1 widthsegs:1 renderable:false
	owoBox.pos = cam.pos
	owoBox.pivot = [0,0,0]

	return owoBox
)
fn collapseOptimizer obj dist stats:"" =
(
local optimizedVerts_arry = #()

	
if (classof obj) == Editable_Poly and obj.verts.count >0 then
(
start = timeStamp()
progressStart (stats[1] +" of "+stats[2]+"->"+obj.name)
perc = 0.0
	
	polyop.setmapsupport obj 0 off
	
	obj.DeleteIsoMapVerts()
	obj.deleteIsoVerts()
	polyOp.collapseDeadStructs obj
	
	if not polyop.getmapsupport obj 0 do polyop.setmapsupport obj 0 on
	polyop.setNumMapVerts obj 0 obj.verts.count
	
	for v in (obj.verts as bitarray) do setVertexColor obj v (color 255 255 255)

	while ((polyop.getVertsByColor obj (color 255 255 255) 0 0 0) as array).count != 0 do
	(
	restVerts = (polyop.getVertsByColor obj (color 255 255 255) 0 0 0) as array
	rValue = random 1 restVerts.count
	rVert = restVerts[rValue]

			if (findItem (polyop.getVertsByColor obj (color 255 0 0) 0 0 0) rVert) == 0 then 
			(
			 polyop.setVertSelection obj rVert
			 optVerts_barry = (polyop.getVertsByColor obj (color 255 0 0) 0 0 0)
				
				for g = 1 to 10 do
				(
				obj.GrowSelection selLevel:1
				selVerts_barry = obj.selectedVerts as bitarray
				selVertsClust_barry = selVerts_barry - optVerts_barry
					

					rVertPos = polyop.getVert obj rVert
					selVertsClustDist_arry = #()
					selVertsClust_arry = selVertsClust_barry as array

					for v = 1 to selVertsClust_arry.count do
					(
						vertPos = polyop.getVert obj selVertsClust_arry[v]
						if (distance rVertPos vertPos) <= dist then append selVertsClustDist_arry selVertsClust_arry[v]
					)
					selVertsClust_barry = (selVertsClustDist_arry as bitarray)

					
				polyop.setVertSelection obj selVertsClust_barry
				)

			selVerts_barry = obj.selectedVerts as bitarray
			selVertsClust_barry = selVerts_barry - (optimizedVerts_arry as bitarray)

				
			polyop.collapseVerts obj selVertsClust_barry
			
			obj.deleteIsoVerts()
			polyOp.collapseDeadStructs obj
			obj.DeleteIsoMapVerts()	
			
			for i = 1 to obj.selectedVerts.count do polyop.setVertColor obj 0 obj.selectedVerts[i].index (color 255 0 0)  --some verts can be Iso!
	
			perc = (((polyop.getVertsByColor obj (color 255 0 0) 0 0 0) as array).count)/(obj.verts.count/100.0) 
			if (progressUpdate perc) == false then exit
			)

	)
progressEnd()
)
	
)

rollout  OWO360_Rollout ("OWO360 - virtual tour 3D - Version "+OWOVer) width:408 height:520
(
	edittext 'edt_OWOFolderPath' "" pos:[32,32] width:264 height:32 align:#left
	button 'btn_setOWOFolder' "..." pos:[304,32] width:64 height:32 align:#left
	GroupBox 'grp1' "Project Folder" pos:[24,8] width:360 height:88 align:#left
	button 'btn_createOWOCamera' "Add OWO Camera" pos:[40,144] width:328 height:40 align:#left
	button 'btn_exportScene' "Export Geometry" pos:[40,216] width:328 height:40 enabled:false align:#left
	button 'btn_renderOWO' "Render 360" pos:[40,280] width:328 height:40 enabled:false align:#left
	button 'btn_OWOTour' "Create Virtual Tour" pos:[41,376] width:328 height:40 enabled:false align:#left
	button 'btn_help' "Help" pos:[248,440] width:120 height:40 align:#left 
 	GroupBox 'grp6' "Scene Settings" pos:[24,112] width:360 height:224 align:#left
	GroupBox 'grp8' "Copy the project files to the server" pos:[24,352] width:360 height:144 align:#left


on btn_help pressed do
(
shellLaunch "http://owo360.ru/tutorial-3ds-max-owo360/" ""	
)	
	
on btn_setOWOFolder pressed do
(
OWOFolder = getSavePath()
	if OWOFolder != undefined then 
	(	
	edt_OWOFolderPath.text = OWOFolder
	btn_renderOWO.enabled = true
	btn_exportScene.enabled = true
	btn_OWOTour.enabled = true 
	)
)
on btn_OWOTour pressed do
(
	if OWOFolder != undefined then shellLaunch "explorer.exe" OWOFolder
	shellLaunch "http://owo360.ru/user/" ""
)

on btn_createOWOCamera pressed do
(
createOWOLayer()

cam = createOWOCamera OWOCamTrans	

if cam != undefined then 
(	
addObjsToOWOLayer #(cam)
select cam
completeredraw()
messageBox ("Camera "+cam.name+" created!")  title:"OWO360"
)

)
on btn_renderOWO pressed do
(
owoCams =#()
setFBXParams()

owoCams = $'owo_Camera_*' as array
	
	if owoCams != undefined then 
	(
		for c = 1 to owoCams.count do
		(
		addOWOToBatch owoCams[c] OWOFolder
		)
	actionMan.executeAction -43434444 "4096"  -- Render: Batch Render Dialog Toggle
	)
)
on btn_exportScene pressed do
(
local doIt = false

doIt = queryBox "Export ALL visible geometry will take some time. Scene will be HOLD. \nContinue?\n " title:"OWO360"

if doIt == true then
(
holdMaxFile()
	
with redraw off
	undo off
	(
	if (getNodeByName "geometry") == true then delete (getNodeByName "geometry")
		
	preparedObjects = for obj in objects where superClassOf obj == geometryClass and  canConvertTo obj Editable_Mesh and obj.ishidden == false collect obj

	maxOps.CloneNodes preparedObjects offset:[0,0,0] expandHierarchy:true cloneType:#copy  newNodes:&owoObjects

	for obj in owoObjects where isValidNode obj do explodeGroup obj

		owoObjectsSel=#()
		o = 0
	
		for obj in owoObjects where isValidNode obj do 
		(	
		o=o+1
		obj.parent = undefined
			
		removeModifier obj TurboSmooth
		removeModifier obj Opensubdiv
		removeModifier obj Tessellate
		removeModifier obj Uvwmap
		removeModifier obj Unwrap_UVW
		
		if (InstanceMgr.CanMakeObjectsUnique obj) == true then InstanceMgr.MakeObjectsUnique obj #individual 
		convertToMesh obj
			
		objsCount = #((o as string),(owoObjects.count as string))
			
		optLvl = undefined
			
		if obj.verts.count < 500 and optLvl == undefined then optLvl = 0
		if obj.verts.count > 500 and obj.verts.count < 5000 and optLvl == undefined then optLvl = 1
		if obj.verts.count > 5000 and obj.verts.count < 10000 and optLvl == undefined then optLvl = 2
		if obj.verts.count > 10000 and obj.verts.count < 20000 and optLvl == undefined then optLvl = 3
		if obj.verts.count > 20000 and obj.verts.count < 30000 and optLvl == undefined then optLvl = 4
		if obj.verts.count > 30000 and optLvl == undefined then optLvl = 5
			
		if optLvl != undefined then 
		(	
			if optLvl == 0 then 
			(
			append owoObjectsSel obj
			)
			
			if optLvl != 0 and optLvl != 5 then 
			(
			addmodifier obj (optimize facethreshold1:paramOptimizeFThresh[optLvl]) 
		
			convertToPoly obj
			polyop.setVertSelection obj #{}
			polyOp.collapseDeadStructs obj
			
			collapseOptimizer obj dist stats:objsCount
			convertToMesh obj 
			append owoObjectsSel obj
			)
			
			if optLvl == 5 then 
			(
			convObj = nvpx.CreateBoundingShapeConvex obj 400 0.5 1
			append owoObjectsSel convObj
			)
		)	
		)--end for



		if owoObjectsSel.count > 0 then 
		(
		select owoObjectsSel


		collapseUtilSelected()
		owoGeom = selection[1]
		owoGeom.name = "geometry"
		owoGeom.material = standard()
		owoGeom.pivot = [0,0,0]
		owoGeom.renderable = false
		createOWOLayer()
		addObjsToOWOLayer #(owoGeom)
			
		fbxFileName = owoGeom.name
		exportToFBX (OWOFolder+"\\"+fbxFileName+".fbx")
		)
	)
	


	
owoCams = $'owo_Camera_*' as array
	
	with redraw off
	undo off
	(
		if owoCams != undefined then 
		(
			for c = 1 to owoCams.count do
			(
			owoBox = createOWOBox owoCams[c]
			select owoBox
			fbxFileName = substring owoCams[c].name 12 owoCams[c].name.count
			exportToFBX (OWOFolder+"\\"+fbxFileName+".fbx")
			delete owoBox
			)
		)
	)
	
)--end doIt
)
on OWO360_Rollout open do
(
--OWODoIt	= checkValidUnits()
	
--	if OWODoIt == false then messageBox "Set the units to millimeters. More info" title:"OWO360"

OWOFolder = OWOFolder_check()
	
	if OWOFolder != undefined then 
	(	
	edt_OWOFolderPath.text = OWOFolder
	btn_renderOWO.enabled = true
	btn_exportScene.enabled = true
	btn_OWOTour.enabled = true 
	)
	
)
)--end rollout

createDialog OWO360_Rollout 	

)--end macro

try (menuMan.unRegisterMenu (menuMan.findMenu "OWO360") ) catch() 

cid = genclassid returnValue:true

if menuMan.registerMenuContext cid[1] then
(
local mainMenuBar = menuMan.getMainMenuBar()
local owo360Menu = menuMan.createMenu "OWO360"
local owo360Item = menuMan.createActionItem "OWO360Tool" "OWO360"
owo360Menu.addItem owo360Item -1
local subMenuItem = menuMan.createSubMenuItem "OWO360" owo360Menu
local subMenuIndex = mainMenuBar.numItems() + 1
mainMenuBar.addItem subMenuItem subMenuIndex
menuMan.updateMenuBar()
)


)

